/******************************************************************
Created with PROGRAMINO IDE for Arduino
Project     : Space Shooter with TFT display
Board       : MAKERFACTORY EBB ESP32
Version     : 1.0
Date        : 14.03.2019
Author      : MF

Description : play the classic Pong game.

Switches:
POTENTIOMETER   SW300 ON
POTENTIOMETER   SW302 ON
I2C PULL-UP     SW307 ON
I2C PULL-UP     SW308 ON
LCD             SW309 all ON
MENU            SW402 Pin 1 OFF
                      Pin 2 OFF
                      Pin 3 OFF
                      Pin 4 ON
                      Pin 5 OFF
                      Pin 6 OFF

Pong game for SH1106 OLED 128x64 display
2018 - Gord Payne
based on work of Ben Wohrl 
https://medium.com/@ruction/arduino-pong-i2c-oled-128x64-bd4185970b50

******************************************************************/

#include <Wire.h>                   // This library allows you to communicate with I2C
#include "SSD1306Wire.h"            // https://github.com/ThingPulse/esp8266-oled-ssd1306
#include "fontovi.h"

// Define the PINS you're goint to use on your Arduino Nano
int controller1 = 39;               // ANALOG 2
int controller2 = 36;               // ANALOG 3
int ledPin      = 13;               // DIGITAL 13
int btnPin      = 27;               // DIGITAL 5
int piezo       = 9;                // buzzer

// Define variables
int buttonState      = 0;           // HIGH = Pressed
int gameState        = 0;           // 0 = Home, 1 = Game, 2 = End
int winSound         = 0;
int controllerValue1 = 0;           // variable to store the value coming from the potentiometer
int controllerValue2 = 0;           // variable to store the value coming from the potentiometer

int paddlePositionPlayer1 = 0;
int paddlePositionPlayer2 = 0;

int scorePlayer1 = 0;
int scorePlayer2 = 0;

int ballX      = 128 / 2;
int ballY      = 64 / 2;
int ballSpeedX = 2;
int ballSpeedY = 1;
int maxScore   = 8;

SSD1306Wire  oled(0x3c, 19, 4);

void setup() 
{
    pinMode(ledPin, OUTPUT);
    pinMode(btnPin, INPUT_PULLUP);
    pinMode(piezo, OUTPUT);
    oled.init();
    oled.flipScreenVertically();
    oled.setFont(ArialMT_Plain_10);
}

void loop() 
{
    oled.clear();
    
    // Read controller value and calculate paddle position
    controllerValue1      = analogRead(controller1);
    controllerValue2      = analogRead(controller2);
    paddlePositionPlayer1 = controllerValue1 * (46.0 / 4096.0);
    paddlePositionPlayer2 = controllerValue2 * (46.0 / 4096.0);
    
    // Set button state
    buttonState = digitalRead(btnPin);
    Serial.println(buttonState);
    if (buttonState == LOW && gameState == 0) 
    {
        gameState = 1;
        delay(150);
    } 
    else if (buttonState == LOW && (gameState == 1 || gameState == 2)) 
    {
        gameState    = 0; // reset for new game
        scorePlayer1 = 0;
        scorePlayer2 = 0;
        ballX        = 128 / 2;
        ballY        = 64 / 2;
        winSound     = 0;
        delay(150);
    }
    
    // display opening screen
    if (gameState == 0) 
    {
        oled.setFont(ArialMT_Plain_10);
        oled.setColor(WHITE);
        oled.drawString(40, 18,"PONG");
        oled.drawString(32, 38,"press start");
    }
    
    // game playing
    if (gameState == 1) 
    {
        drawField(scorePlayer1, scorePlayer2); // update screen
        collisionControl(); // check for collision
        drawBall();
    }
    
    if (gameState == 2) 
    {
        drawField(scorePlayer1, scorePlayer2);
        if (scorePlayer1 == maxScore) 
        {
            oled.drawString(15, 30, "winner!");
        } 
        else if (scorePlayer2 == maxScore) 
        {
            oled.drawString(77, 30, "winner!");
        }
    }
    
    oled.display();
}

void drawField(int score1, int score2) 
{
    oled.fillRect(0, round(paddlePositionPlayer1), 2, 18);
    oled.fillRect(126, round(paddlePositionPlayer2), 2, 18);
    oled.setFont(ArialMT_Plain_10);
    oled.setColor(WHITE);
    oled.drawString(54,10,String(score1));
    oled.drawString(68,10,String(score2));
    oled.fillRect(63, 12, 1, 5);
    oled.fillRect(63, 22, 1, 5);
    oled.fillRect(63, 32, 1, 5);
    oled.fillRect(63, 42, 1, 5);
    oled.fillRect(63, 52, 1, 5);
    oled.fillRect(63, 62, 1, 5);
}

void collisionControl() 
{
    // bounce from top and bottom
    if (ballY >= 64 - 2 || ballY <= 0) 
    {
        ballSpeedY *= -1;
    }
    
    // score points if ball hits wall behind player
    if (ballX >= 128 - 2 || ballX <= 0) 
    {
        if (ballSpeedX > 0) 
        {
            scorePlayer1++;
            ballX = 128 / 4;
        }
        
        if (ballSpeedX < 0) 
        {
            scorePlayer2++;
            ballX = 128 / 4 * 3;
        }
        
        if (scorePlayer1 == maxScore || scorePlayer2 == maxScore) 
        {
            gameState = 2;
        }
    }
    
    // bounce from player1
    if (ballX >= 0 && ballX <= 2 && ballSpeedX < 0) 
    {
        if (ballY > round(paddlePositionPlayer1) - 2 && ballY < round(paddlePositionPlayer1) + 18) 
        {
            ballSpeedX *= -1;
        }
    }
    
    // bounce from player2
    if (ballX >= 128 - 2 - 2 && ballX <= 128 - 2 && ballSpeedX > 0) 
    {
        if (ballY > round(paddlePositionPlayer2) - 2 && ballY < round(paddlePositionPlayer2) + 18) 
        {
            ballSpeedX *= -1;
        }
    }
}

void drawBall() 
{
    oled.fillRect(ballX, ballY, 2, 2);
    ballX += ballSpeedX;
    ballY += ballSpeedY;
    delay(20);
}
